from __future__ import print_function
import numpy as np


class Obj:

  def __init__(self, fn):
    self.ind_v = 0
    self.ind_vt = 0
    self.ind_vn = 0
    self.fn = fn
    self.out = open(fn + ".tmp", "w")
    self.out.write("mtllib dinnerware.mtl\n")

  def __del__(self):
    self.out.close()
    import shutil
    shutil.move(self.fn + ".tmp", self.fn)

  def push_v(self, v):
    self.out.write("v %f %f %f\n" % (v[0], v[1], v[2]))
    self.ind_v += 1
    return self.ind_v

  def push_vt(self, vt):
    self.out.write("vt %f %f\n" % (vt[0], vt[1]))
    self.ind_vt += 1
    return self.ind_vt

  def push_vn(self, vn):
    vn /= np.linalg.norm(vn)
    self.out.write("vn %f %f %f\n" % (vn[0], vn[1], vn[2]))
    self.ind_vn += 1
    return self.ind_vn


def convex_hull(points, vind, nind, tind, obj):
  "super ineffective"
  cnt = len(points)
  for a in range(cnt):
    for b in range(a + 1, cnt):
      for c in range(b + 1, cnt):
        vec1 = points[a] - points[b]
        vec2 = points[a] - points[c]
        n = np.cross(vec1, vec2)
        n /= np.linalg.norm(n)
        C = np.dot(n, points[a])
        inner = np.inner(n, points)
        pos = (inner <= C + 0.0001).all()
        neg = (inner >= C - 0.0001).all()
        if not pos and not neg: continue
        obj.out.write("f %i//%i %i//%i %i//%i\n" %
                      ((vind[a], nind[a], vind[b], nind[b], vind[c], nind[c]) if
                       (inner - C).sum() < 0 else
                       (vind[a], nind[a], vind[c], nind[c], vind[b], nind[b])))
        #obj.out.write("f %i/%i/%i %i/%i/%i %i/%i/%i\n" % (
        #	(vind[a], tind[a], nind[a], vind[b], tind[b], nind[b], vind[c], tind[c], nind[c])
        #	if (inner - C).sum() < 0 else
        #	(vind[a], tind[a], nind[a], vind[c], tind[c], nind[c], vind[b], tind[b], nind[b]) ) )


def test_convex_hull():
  obj = Obj("convex_test.obj")
  vlist = np.random.uniform(low=-0.1, high=+0.1, size=(100, 3))
  nlist = vlist.copy()
  tlist = np.random.uniform(low=0, high=+1, size=(100, 2))
  vind = [obj.push_v(xyz) for xyz in vlist]
  nind = [obj.push_vn(xyz) for xyz in nlist]
  tind = [obj.push_vt(uv) for uv in tlist]
  convex_hull(vlist, vind, nind, tind, obj)


class Contour:

  def __init__(self):
    self.vprev_vind = None

  def f(self, obj, vlist_vind, vlist_tind, vlist_nind):
    cnt = len(vlist_vind)
    for i1 in range(cnt):
      i2 = i1 - 1
      obj.out.write("f %i/%i/%i %i/%i/%i %i/%i/%i\n" % (
          vlist_vind[i2],
          vlist_tind[i2],
          vlist_nind[i2],
          vlist_vind[i1],
          vlist_tind[i1],
          vlist_nind[i1],
          self.vprev_vind[i1],
          self.vprev_tind[i1],
          self.vprev_nind[i1],
      ))
      obj.out.write("f %i/%i/%i %i/%i/%i %i/%i/%i\n" % (
          vlist_vind[i2],
          vlist_tind[i2],
          vlist_nind[i2],
          self.vprev_vind[i1],
          self.vprev_tind[i1],
          self.vprev_nind[i1],
          self.vprev_vind[i2],
          self.vprev_tind[i2],
          self.vprev_nind[i2],
      ))

  def belt(self, obj, vlist, nlist, tlist):
    vlist_vind = [obj.push_v(xyz) for xyz in vlist]
    vlist_tind = [obj.push_vt(xyz) for xyz in tlist]
    vlist_nind = [obj.push_vn(xyz) for xyz in nlist]
    if self.vprev_vind:
      self.f(obj, vlist_vind, vlist_tind, vlist_nind)
    else:
      self.first_vind = vlist_vind
      self.first_tind = vlist_tind
      self.first_nind = vlist_nind
    self.vprev_vind = vlist_vind
    self.vprev_tind = vlist_tind
    self.vprev_nind = vlist_nind

  def finish(self, obj):
    self.f(obj, self.first_vind, self.first_tind, self.first_nind)


def test_contour():
  RAD1 = 2.0
  RAD2 = 1.5
  obj = Obj("torus.obj")
  obj.out.write("usemtl porcelain\n")
  contour = Contour()
  for step in range(100):
    angle = step / 100.0 * 2 * np.pi
    belt_v = []
    belt_n = []
    belt_t = []
    for b in range(50):
      beta = b / 50.0 * 2 * np.pi
      r = RAD2 * np.cos(beta) + RAD1
      z = RAD2 * np.sin(beta)
      belt_v.append(np.array([np.cos(angle) * r, np.sin(angle) * r, z]))
      belt_n.append(
          np.array([np.cos(angle) * np.cos(beta),
                    np.sin(angle) * np.cos(beta),
                    np.sin(beta)]))
      belt_t.append((0, 0))
    contour.belt(obj, belt_v, belt_n, belt_t)
  contour.finish(obj)


#test_convex_hull()
#test_contour()


class RotationFigureParams:
  pass


def generate_plate(p, obj, collision_prefix):
  contour = Contour()
  belt_vlist_3d_prev = None

  for step in range(p.N_VIZ + 1):
    angle = step / float(p.N_VIZ) * 2 * np.pi

    if step % p.COLLISION_EVERY == 0:
      vlist_3d = []
      for x, y in p.belt_simple:
        vlist_3d.append([np.cos(angle) * x * 1.06, np.sin(angle) * x * 1.06, y])
      if belt_vlist_3d_prev:
        obj2 = Obj(collision_prefix % (step / p.COLLISION_EVERY))
        obj2.out.write("usemtl pan_tefal\n")
        vlist = np.array(vlist_3d + belt_vlist_3d_prev)
        vlist[len(vlist_3d):] *= 1.01  # break points on one plane
        vlist[0, 0:2] += 0.01 * vlist[len(vlist_3d), 0:2]
        vlist[len(vlist_3d), 0:2] += 0.01 * vlist[0, 0:2]
        nlist = np.random.uniform(low=-1, high=+1, size=vlist.shape)
        tlist = np.random.uniform(low=0, high=+1, size=(len(vlist), 2))
        vind = [obj2.push_v(xyz) for xyz in vlist]
        nind = [obj2.push_vn(xyz) for xyz in nlist]
        convex_hull(vlist, vind, nind, None, obj2)
      belt_vlist_3d_prev = vlist_3d
    if step == p.N_VIZ: break

    belt_v = []
    belt_n = []
    belt_t = []
    for x, y, nx, ny in p.belt:
      belt_v.append(np.array([np.cos(angle) * x, np.sin(angle) * x, y]))
      belt_n.append(np.array([np.cos(angle) * nx, np.sin(angle) * nx, ny]))
      if ny - nx >= 0:
        belt_t.append((127.0 / 512 + np.cos(angle) * x / p.RAD_HIGH * 105 / 512,
                       (512 - 135.0) / 512 + np.sin(angle) * x / p.RAD_HIGH * 105 / 512))
      else:
        belt_t.append((382.0 / 512 + np.cos(angle) * x / p.RAD_HIGH * 125 / 512,
                       (512 - 380.0) / 512 + np.sin(angle) * x / p.RAD_HIGH * 125 / 512))
    contour.belt(obj, belt_v, belt_n, belt_t)

  contour.finish(obj)


def tefal():
  p = RotationFigureParams()
  p.RAD_LOW = 0.240 / 2
  p.RAD_HIGH = 0.255 / 2
  p.H = 0.075
  p.THICK = 0.005
  p.N_VIZ = 30
  p.COLLISION_EVERY = 5
  p.belt = [
      (p.RAD_HIGH - p.THICK, p.H, -1, 0),  # x y norm
      (p.RAD_HIGH, p.H, 0, 1),
      (p.RAD_HIGH + p.THICK, p.H, +1, 0),
      (p.RAD_LOW + p.THICK, p.THICK, +1, 0),
      (p.RAD_LOW, 0, 0, -1),
      (0, 0, 0, -1),
      (0, p.THICK, 0, 1),
      (p.RAD_LOW - p.THICK, p.THICK, 0, 1),
      (p.RAD_LOW - p.THICK, 3 * p.THICK, -1, 0),
  ]
  p.belt.reverse()
  p.belt_simple = [(p.RAD_HIGH - p.THICK, p.H), (p.RAD_HIGH + p.THICK, p.H), (p.RAD_LOW, 0),
                   (p.RAD_LOW - p.THICK, 0)]
  obj = Obj("pan_tefal.obj")
  obj.out.write("usemtl pan_tefal\n")
  generate_plate(p, obj, "pan_tefal-collision%02i.obj")


def plate():
  p = RotationFigureParams()
  p.RAD_LOW = 0.110 / 2
  p.RAD_HIGH = 0.190 / 2
  p.H = 0.060
  p.THICK = 0.003
  p.N_VIZ = 30
  p.COLLISION_EVERY = 5
  p.belt = [
      (p.RAD_HIGH - p.THICK, p.H, -0.9, 0.5),  # x y norm
      (p.RAD_HIGH, p.H, 0, 1),
      (p.RAD_HIGH + p.THICK, p.H, +1, 0),
      (p.RAD_LOW + p.THICK, p.THICK, +1, 0),
      (p.RAD_LOW, 0, 0, -1),
      (0, 0, 0, -1),
      (0, p.THICK, 0, 1),
      (p.RAD_LOW - 3 * p.THICK, p.THICK, 0, 1),
      (p.RAD_LOW - p.THICK, 3 * p.THICK, -0.5, 1.0),
  ]
  p.belt.reverse()
  p.belt_simple = [(p.RAD_HIGH - p.THICK, p.H), (p.RAD_HIGH + p.THICK, p.H), (p.RAD_LOW, 0),
                   (p.RAD_LOW - p.THICK, 0)]
  obj = Obj("plate.obj")
  obj.out.write("usemtl solid_color\n")
  generate_plate(p, obj, "plate-collision%02i.obj")


plate()



import pybullet as p
import time
import math

pi = 3.14159264
limitVal = 2 * pi
legpos = 3. / 4. * pi
legposS = 0
legposSright = 0  #-0.3
legposSleft = 0  #0.3

defaultERP = 0.4
maxMotorForce = 5000
maxGearForce = 10000
jointFriction = 0.1

p.connect(p.GUI)

amplitudeId = p.addUserDebugParameter("amplitude", 0, 3.14, 0.5)
timeScaleId = p.addUserDebugParameter("timeScale", 0, 10, 1)

jointTypeNames = {}
jointTypeNames[p.JOINT_REVOLUTE] = "JOINT_REVOLUTE"
jointTypeNames[p.JOINT_FIXED] = "JOINT_FIXED"
p.setPhysicsEngineParameter(numSolverIterations=100)
p.loadURDF("plane_transparent.urdf", useMaximalCoordinates=True)
#disable rendering during creation.
#p.configureDebugVisualizer(p.COV_ENABLE_RENDERING,1)
p.configureDebugVisualizer(p.COV_ENABLE_PLANAR_REFLECTION, 1)

jointNamesToIndex = {}

p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, 0)
vision = p.loadURDF("vision60.urdf", [0, 0, 0.4], useFixedBase=False)
p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, 1)

for j in range(p.getNumJoints(vision)):
  jointInfo = p.getJointInfo(vision, j)
  jointInfoName = jointInfo[1].decode("utf-8")
  print("joint ", j, " = ", jointInfoName, "type=", jointTypeNames[jointInfo[2]])
  jointNamesToIndex[jointInfoName] = j
  #print("jointNamesToIndex[..]=",jointNamesToIndex[jointInfoName])
  p.setJointMotorControl2(vision, j, p.VELOCITY_CONTROL, targetVelocity=0, force=jointFriction)

chassis_right_center = jointNamesToIndex['chassis_right_center']
motor_front_rightR_joint = jointNamesToIndex['motor_front_rightR_joint']
motor_front_rightS_joint = jointNamesToIndex['motor_front_rightS_joint']
hip_front_rightR_joint = jointNamesToIndex['hip_front_rightR_joint']
knee_front_rightR_joint = jointNamesToIndex['knee_front_rightR_joint']
motor_front_rightL_joint = jointNamesToIndex['motor_front_rightL_joint']
motor_back_rightR_joint = jointNamesToIndex['motor_back_rightR_joint']
motor_back_rightS_joint = jointNamesToIndex['motor_back_rightS_joint']
hip_back_rightR_joint = jointNamesToIndex['hip_back_rightR_joint']
knee_back_rightR_joint = jointNamesToIndex['knee_back_rightR_joint']
motor_back_rightL_joint = jointNamesToIndex['motor_back_rightL_joint']
chassis_left_center = jointNamesToIndex['chassis_left_center']
motor_front_leftL_joint = jointNamesToIndex['motor_front_leftL_joint']
motor_front_leftS_joint = jointNamesToIndex['motor_front_leftS_joint']
hip_front_leftL_joint = jointNamesToIndex['hip_front_leftL_joint']
knee_front_leftL_joint = jointNamesToIndex['knee_front_leftL_joint']
motor_front_leftR_joint = jointNamesToIndex['motor_front_leftR_joint']
motor_back_leftL_joint = jointNamesToIndex['motor_back_leftL_joint']
hip_back_leftL_joint = jointNamesToIndex['hip_back_leftL_joint']
knee_back_leftL_joint = jointNamesToIndex['knee_back_leftL_joint']
motor_back_leftR_joint = jointNamesToIndex['motor_back_leftR_joint']
motor_back_leftS_joint = jointNamesToIndex['motor_back_leftS_joint']

motA_rf_Id = p.addUserDebugParameter("motA_rf", -limitVal, limitVal, legpos)
motB_rf_Id = p.addUserDebugParameter("motB_rf", -limitVal, limitVal, legpos)
motC_rf_Id = p.addUserDebugParameter("motC_rf", -limitVal, limitVal, legposSright)
erp_rf_Id = p.addUserDebugParameter("erp_rf", 0, 1, defaultERP)
relPosTarget_rf_Id = p.addUserDebugParameter("relPosTarget_rf", -limitVal, limitVal, 0)

motA_lf_Id = p.addUserDebugParameter("motA_lf", -limitVal, limitVal, -legpos)
motB_lf_Id = p.addUserDebugParameter("motB_lf", -limitVal, limitVal, -legpos)
motC_lf_Id = p.addUserDebugParameter("motC_lf", -limitVal, limitVal, legposSleft)

erp_lf_Id = p.addUserDebugParameter("erp_lf", 0, 1, defaultERP)
relPosTarget_lf_Id = p.addUserDebugParameter("relPosTarget_lf", -limitVal, limitVal, 0)

motA_rb_Id = p.addUserDebugParameter("motA_rb", -limitVal, limitVal, legpos)
motB_rb_Id = p.addUserDebugParameter("motB_rb", -limitVal, limitVal, legpos)
motC_rb_Id = p.addUserDebugParameter("motC_rb", -limitVal, limitVal, legposSright)

erp_rb_Id = p.addUserDebugParameter("erp_rb", 0, 1, defaultERP)
relPosTarget_rb_Id = p.addUserDebugParameter("relPosTarget_rb", -limitVal, limitVal, 0)

motA_lb_Id = p.addUserDebugParameter("motA_lb", -limitVal, limitVal, -legpos)
motB_lb_Id = p.addUserDebugParameter("motB_lb", -limitVal, limitVal, -legpos)
motC_lb_Id = p.addUserDebugParameter("motC_lb", -limitVal, limitVal, legposSleft)

erp_lb_Id = p.addUserDebugParameter("erp_lb", 0, 1, defaultERP)
relPosTarget_lb_Id = p.addUserDebugParameter("relPosTarget_lb", -limitVal, limitVal, 0)

camTargetPos = [0.25, 0.62, -0.15]
camDist = 2
camYaw = -2
camPitch = -16
p.resetDebugVisualizerCamera(camDist, camYaw, camPitch, camTargetPos)

c_rf = p.createConstraint(vision,
                          knee_front_rightR_joint,
                          vision,
                          motor_front_rightL_joint,
                          jointType=p.JOINT_GEAR,
                          jointAxis=[0, 1, 0],
                          parentFramePosition=[0, 0, 0],
                          childFramePosition=[0, 0, 0])
p.changeConstraint(c_rf, gearRatio=-1, gearAuxLink=motor_front_rightR_joint, maxForce=maxGearForce)

c_lf = p.createConstraint(vision,
                          knee_front_leftL_joint,
                          vision,
                          motor_front_leftR_joint,
                          jointType=p.JOINT_GEAR,
                          jointAxis=[0, 1, 0],
                          parentFramePosition=[0, 0, 0],
                          childFramePosition=[0, 0, 0])
p.changeConstraint(c_lf, gearRatio=-1, gearAuxLink=motor_front_leftL_joint, maxForce=maxGearForce)

c_rb = p.createConstraint(vision,
                          knee_back_rightR_joint,
                          vision,
                          motor_back_rightL_joint,
                          jointType=p.JOINT_GEAR,
                          jointAxis=[0, 1, 0],
                          parentFramePosition=[0, 0, 0],
                          childFramePosition=[0, 0, 0])
p.changeConstraint(c_rb, gearRatio=-1, gearAuxLink=motor_back_rightR_joint, maxForce=maxGearForce)

c_lb = p.createConstraint(vision,
                          knee_back_leftL_joint,
                          vision,
                          motor_back_leftR_joint,
                          jointType=p.JOINT_GEAR,
                          jointAxis=[0, 1, 0],
                          parentFramePosition=[0, 0, 0],
                          childFramePosition=[0, 0, 0])
p.changeConstraint(c_lb, gearRatio=-1, gearAuxLink=motor_back_leftL_joint, maxForce=maxGearForce)

p.setRealTimeSimulation(1)
for i in range(1):
  #while (1):
  motA_rf = p.readUserDebugParameter(motA_rf_Id)
  motB_rf = p.readUserDebugParameter(motB_rf_Id)
  motC_rf = p.readUserDebugParameter(motC_rf_Id)
  erp_rf = p.readUserDebugParameter(erp_rf_Id)
  relPosTarget_rf = p.readUserDebugParameter(relPosTarget_rf_Id)
  #motC_rf
  p.setJointMotorControl2(vision,
                          motor_front_rightR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_rf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_rightL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_rf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_rightS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_rf,
                          force=maxMotorForce)
  p.changeConstraint(c_rf,
                     gearRatio=-1,
                     gearAuxLink=motor_front_rightR_joint,
                     erp=erp_rf,
                     relativePositionTarget=relPosTarget_rf,
                     maxForce=maxGearForce)

  motA_lf = p.readUserDebugParameter(motA_lf_Id)
  motB_lf = p.readUserDebugParameter(motB_lf_Id)
  motC_lf = p.readUserDebugParameter(motC_lf_Id)
  erp_lf = p.readUserDebugParameter(erp_lf_Id)
  relPosTarget_lf = p.readUserDebugParameter(relPosTarget_lf_Id)
  p.setJointMotorControl2(vision,
                          motor_front_leftL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_lf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_leftR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_lf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_leftS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_lf,
                          force=maxMotorForce)
  p.changeConstraint(c_lf,
                     gearRatio=-1,
                     gearAuxLink=motor_front_leftL_joint,
                     erp=erp_lf,
                     relativePositionTarget=relPosTarget_lf,
                     maxForce=maxGearForce)

  motA_rb = p.readUserDebugParameter(motA_rb_Id)
  motB_rb = p.readUserDebugParameter(motB_rb_Id)
  motC_rb = p.readUserDebugParameter(motC_rb_Id)
  erp_rb = p.readUserDebugParameter(erp_rb_Id)
  relPosTarget_rb = p.readUserDebugParameter(relPosTarget_rb_Id)
  p.setJointMotorControl2(vision,
                          motor_back_rightR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_rb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_rightL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_rb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_rightS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_rb,
                          force=maxMotorForce)
  p.changeConstraint(c_rb,
                     gearRatio=-1,
                     gearAuxLink=motor_back_rightR_joint,
                     erp=erp_rb,
                     relativePositionTarget=relPosTarget_rb,
                     maxForce=maxGearForce)

  motA_lb = p.readUserDebugParameter(motA_lb_Id)
  motB_lb = p.readUserDebugParameter(motB_lb_Id)
  motC_lb = p.readUserDebugParameter(motC_lb_Id)
  erp_lb = p.readUserDebugParameter(erp_lb_Id)
  relPosTarget_lb = p.readUserDebugParameter(relPosTarget_lb_Id)
  p.setJointMotorControl2(vision,
                          motor_back_leftL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_lb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_leftR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_lb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_leftS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_lb,
                          force=maxMotorForce)
  p.changeConstraint(c_lb,
                     gearRatio=-1,
                     gearAuxLink=motor_back_leftL_joint,
                     erp=erp_lb,
                     relativePositionTarget=relPosTarget_lb,
                     maxForce=maxGearForce)

  p.setGravity(0, 0, -10)
  time.sleep(1. / 240.)
t = 0
prevTime = time.time()
while (1):
  timeScale = p.readUserDebugParameter(timeScaleId)
  amplitude = p.readUserDebugParameter(amplitudeId)
  newTime = time.time()
  dt = (newTime - prevTime) * timeScale
  t = t + dt
  prevTime = newTime

  amp = amplitude
  motA_rf = math.sin(t) * amp + legpos
  motA_rb = math.sin(t) * amp + legpos
  motA_lf = -(math.sin(t) * amp + legpos)
  motA_lb = -(math.sin(t) * amp + legpos)

  motB_rf = math.sin(t) * amp + legpos
  motB_rb = math.sin(t) * amp + legpos
  motB_lf = -(math.sin(t) * amp + legpos)
  motB_lb = -(math.sin(t) * amp + legpos)

  p.setJointMotorControl2(vision,
                          motor_front_rightR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_rf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_rightL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_rf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_rightS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_rf,
                          force=maxMotorForce)

  p.setJointMotorControl2(vision,
                          motor_front_leftL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_lf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_leftR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_lf,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_front_leftS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_lf,
                          force=maxMotorForce)

  p.setJointMotorControl2(vision,
                          motor_back_rightR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_rb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_rightL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_rb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_rightS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_rb,
                          force=maxMotorForce)

  p.setJointMotorControl2(vision,
                          motor_back_leftL_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motA_lb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_leftR_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motB_lb,
                          force=maxMotorForce)
  p.setJointMotorControl2(vision,
                          motor_back_leftS_joint,
                          p.POSITION_CONTROL,
                          targetPosition=motC_lb,
                          force=maxMotorForce)

  p.setGravity(0, 0, -10)
  time.sleep(1. / 240.)



#! /usr/bin/env python
# Copyright (c) 2013, Willow Garage, Inc.
# Copyright (c) 2014, Open Source Robotics Foundation, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Open Source Robotics Foundation, Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior
#       written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Author: Stuart Glaser
# Maintainer: William Woodall <william@osrfoundation.org>

from __future__ import print_function

import getopt
import glob
import os
import re
import string
import sys
import xml

from xml.dom.minidom import parse

try:
  _basestr = basestring
except NameError:
  _basestr = str

# Dictionary of subtitution args
substitution_args_context = {}


class XacroException(Exception):
  pass


def isnumber(x):
  return hasattr(x, '__int__')


# Better pretty printing of xml
# Taken from http://ronrothman.com/public/leftbraned/xml-dom-minidom-toprettyxml-and-silly-whitespace/
def fixed_writexml(self, writer, indent="", addindent="", newl=""):
  # indent = current indentation
  # addindent = indentation to add to higher levels
  # newl = newline string
  writer.write(indent + "<" + self.tagName)

  attrs = self._get_attributes()
  a_names = list(attrs.keys())
  a_names.sort()

  for a_name in a_names:
    writer.write(" %s=\"" % a_name)
    xml.dom.minidom._write_data(writer, attrs[a_name].value)
    writer.write("\"")
  if self.childNodes:
    if len(self.childNodes) == 1 \
       and self.childNodes[0].nodeType == xml.dom.minidom.Node.TEXT_NODE:
      writer.write(">")
      self.childNodes[0].writexml(writer, "", "", "")
      writer.write("</%s>%s" % (self.tagName, newl))
      return
    writer.write(">%s" % (newl))
    for node in self.childNodes:
      # skip whitespace-only text nodes
      if node.nodeType == xml.dom.minidom.Node.TEXT_NODE and \
         not node.data.strip():
        continue
      node.writexml(writer, indent + addindent, addindent, newl)
    writer.write("%s</%s>%s" % (indent, self.tagName, newl))
  else:
    writer.write("/>%s" % (newl))


# replace minidom's function with ours
xml.dom.minidom.Element.writexml = fixed_writexml


class Table:

  def __init__(self, parent=None):
    self.parent = parent
    self.table = {}

  def __getitem__(self, key):
    if key in self.table:
      return self.table[key]
    elif self.parent:
      return self.parent[key]
    else:
      raise KeyError(key)

  def __setitem__(self, key, value):
    self.table[key] = value

  def __contains__(self, key):
    return \
        key in self.table or \
        (self.parent and key in self.parent)


class QuickLexer(object):

  def __init__(self, **res):
    self.str = ""
    self.top = None
    self.res = []
    for k, v in res.items():
      self.__setattr__(k, len(self.res))
      self.res.append(v)

  def lex(self, str):
    self.str = str
    self.top = None
    self.next()

  def peek(self):
    return self.top

  def next(self):
    result = self.top
    self.top = None
    for i in range(len(self.res)):
      m = re.match(self.res[i], self.str)
      if m:
        self.top = (i, m.group(0))
        self.str = self.str[m.end():]
        break
    return result


def first_child_element(elt):
  c = elt.firstChild
  while c:
    if c.nodeType == xml.dom.Node.ELEMENT_NODE:
      return c
    c = c.nextSibling
  return None


def next_sibling_element(elt):
  c = elt.nextSibling
  while c:
    if c.nodeType == xml.dom.Node.ELEMENT_NODE:
      return c
    c = c.nextSibling
  return None


# Pre-order traversal of the elements
def next_element(elt):
  child = first_child_element(elt)
  if child:
    return child
  while elt and elt.nodeType == xml.dom.Node.ELEMENT_NODE:
    next = next_sibling_element(elt)
    if next:
      return next
    elt = elt.parentNode
  return None


# Pre-order traversal of all the nodes
def next_node(node):
  if node.firstChild:
    return node.firstChild
  while node:
    if node.nextSibling:
      return node.nextSibling
    node = node.parentNode
  return None


def child_nodes(elt):
  c = elt.firstChild
  while c:
    yield c
    c = c.nextSibling


all_includes = []

# Deprecated message for <include> tags that don't have <xacro:include> prepended:
deprecated_include_msg = """DEPRECATED IN HYDRO:
  The <include> tag should be prepended with 'xacro' if that is the intended use
  of it, such as <xacro:include ...>. Use the following script to fix incorrect
  xacro includes:
     sed -i 's/<include/<xacro:include/g' `find . -iname *.xacro`"""

include_no_matches_msg = """Include tag filename spec \"{}\" matched no files."""


## @throws XacroException if a parsing error occurs with an included document
def process_includes(doc, base_dir):
  namespaces = {}
  previous = doc.documentElement
  elt = next_element(previous)
  while elt:
    # Xacro should not use plain 'include' tags but only namespaced ones. Causes conflicts with
    # other XML elements including Gazebo's <gazebo> extensions
    is_include = False
    if elt.tagName == 'xacro:include' or elt.tagName == 'include':

      is_include = True
      # Temporary fix for ROS Hydro and the xacro include scope problem
      if elt.tagName == 'include':

        # check if there is any element within the <include> tag. mostly we are concerned
        # with Gazebo's <uri> element, but it could be anything. also, make sure the child
        # nodes aren't just a single Text node, which is still considered a deprecated
        # instance
        if elt.childNodes and not (len(elt.childNodes) == 1 and
                                   elt.childNodes[0].nodeType == elt.TEXT_NODE):
          # this is not intended to be a xacro element, so we can ignore it
          is_include = False
        else:
          # throw a deprecated warning
          print(deprecated_include_msg, file=sys.stderr)

    # Process current element depending on previous conditions
    if is_include:
      filename_spec = eval_text(elt.getAttribute('filename'), {})
      if not os.path.isabs(filename_spec):
        filename_spec = os.path.join(base_dir, filename_spec)

      if re.search('[*[?]+', filename_spec):
        # Globbing behaviour
        filenames = sorted(glob.glob(filename_spec))
        if len(filenames) == 0:
          print(include_no_matches_msg.format(filename_spec), file=sys.stderr)
      else:
        # Default behaviour
        filenames = [filename_spec]

      for filename in filenames:
        global all_includes
        all_includes.append(filename)
        try:
          with open(filename) as f:
            try:
              included = parse(f)
            except Exception as e:
              raise XacroException(
                  "included file \"%s\" generated an error during XML parsing: %s" %
                  (filename, str(e)))
        except IOError as e:
          raise XacroException("included file \"%s\" could not be opened: %s" % (filename, str(e)))

        # Replaces the include tag with the elements of the included file
        for c in child_nodes(included.documentElement):
          elt.parentNode.insertBefore(c.cloneNode(deep=True), elt)

        # Grabs all the declared namespaces of the included document
        for name, value in included.documentElement.attributes.items():
          if name.startswith('xmlns:'):
            namespaces[name] = value

      elt.parentNode.removeChild(elt)
      elt = None
    else:
      previous = elt

    elt = next_element(previous)

  # Makes sure the final document declares all the namespaces of the included documents.
  for k, v in namespaces.items():
    doc.documentElement.setAttribute(k, v)


# Returns a dictionary: { macro_name => macro_xml_block }
def grab_macros(doc):
  macros = {}

  previous = doc.documentElement
  elt = next_element(previous)
  while elt:
    if elt.tagName == 'macro' or elt.tagName == 'xacro:macro':
      name = elt.getAttribute('name')

      macros[name] = elt
      macros['xacro:' + name] = elt

      elt.parentNode.removeChild(elt)
      elt = None
    else:
      previous = elt

    elt = next_element(previous)
  return macros


# Returns a Table of the properties
def grab_properties(doc):
  table = Table()

  previous = doc.documentElement
  elt = next_element(previous)
  while elt:
    if elt.tagName == 'property' or elt.tagName == 'xacro:property':
      name = elt.getAttribute('name')
      value = None

      if elt.hasAttribute('value'):
        value = elt.getAttribute('value')
      else:
        name = '**' + name
        value = elt  # debug

      bad = string.whitespace + "${}"
      has_bad = False
      for b in bad:
        if b in name:
          has_bad = True
          break

      if has_bad:
        sys.stderr.write('Property names may not have whitespace, ' + '"{", "}", or "$" : "' +
                         name + '"')
      else:
        table[name] = value

      elt.parentNode.removeChild(elt)
      elt = None
    else:
      previous = elt

    elt = next_element(previous)
  return table


def eat_ignore(lex):
  while lex.peek() and lex.peek()[0] == lex.IGNORE:
    lex.next()


def eval_lit(lex, symbols):
  eat_ignore(lex)
  if lex.peek()[0] == lex.NUMBER:
    return float(lex.next()[1])
  if lex.peek()[0] == lex.SYMBOL:
    try:
      key = lex.next()[1]
      value = symbols[key]
    except KeyError as ex:
      raise XacroException("Property wasn't defined: %s" % str(ex))
    if not (isnumber(value) or isinstance(value, _basestr)):
      if value is None:
        raise XacroException("Property %s recursively used" % key)
      raise XacroException("WTF2")
    try:
      return int(value)
    except:
      try:
        return float(value)
      except:
        # prevent infinite recursion
        symbols[key] = None
        result = eval_text(value, symbols)
        # restore old entry
        symbols[key] = value
        return result
  raise XacroException("Bad literal")


def eval_factor(lex, symbols):
  eat_ignore(lex)

  neg = 1
  if lex.peek()[1] == '-':
    lex.next()
    neg = -1

  if lex.peek()[0] in [lex.NUMBER, lex.SYMBOL]:
    return neg * eval_lit(lex, symbols)
  if lex.peek()[0] == lex.LPAREN:
    lex.next()
    eat_ignore(lex)
    result = eval_expr(lex, symbols)
    eat_ignore(lex)
    if lex.next()[0] != lex.RPAREN:
      raise XacroException("Unmatched left paren")
    eat_ignore(lex)
    return neg * result

  raise XacroException("Misplaced operator")


def eval_term(lex, symbols):
  eat_ignore(lex)

  result = 0
  if lex.peek()[0] in [lex.NUMBER, lex.SYMBOL, lex.LPAREN] \
          or lex.peek()[1] == '-':
    result = eval_factor(lex, symbols)

  eat_ignore(lex)
  while lex.peek() and lex.peek()[1] in ['*', '/']:
    op = lex.next()[1]
    n = eval_factor(lex, symbols)

    if op == '*':
      result = float(result) * float(n)
    elif op == '/':
      result = float(result) / float(n)
    else:
      raise XacroException("WTF")
    eat_ignore(lex)
  return result


def eval_expr(lex, symbols):
  eat_ignore(lex)

  op = None
  if lex.peek()[0] == lex.OP:
    op = lex.next()[1]
    if not op in ['+', '-']:
      raise XacroException("Invalid operation. Must be '+' or '-'")

  result = eval_term(lex, symbols)
  if op == '-':
    result = -float(result)

  eat_ignore(lex)
  while lex.peek() and lex.peek()[1] in ['+', '-']:
    op = lex.next()[1]
    n = eval_term(lex, symbols)

    if op == '+':
      result = float(result) + float(n)
    if op == '-':
      result = float(result) - float(n)
    eat_ignore(lex)
  return result


def eval_text(text, symbols):

  def handle_expr(s):
    lex = QuickLexer(IGNORE=r"\s+",
                     NUMBER=r"(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?",
                     SYMBOL=r"[a-zA-Z_]\w*",
                     OP=r"[\+\-\*/^]",
                     LPAREN=r"\(",
                     RPAREN=r"\)")
    lex.lex(s)
    return eval_expr(lex, symbols)

  def handle_extension(s):
    return ("$(%s)" % s)

  results = []
  lex = QuickLexer(DOLLAR_DOLLAR_BRACE=r"\$\$+\{",
                   EXPR=r"\$\{[^\}]*\}",
                   EXTENSION=r"\$\([^\)]*\)",
                   TEXT=r"([^\$]|\$[^{(]|\$$)+")
  lex.lex(text)
  while lex.peek():
    if lex.peek()[0] == lex.EXPR:
      results.append(handle_expr(lex.next()[1][2:-1]))
    elif lex.peek()[0] == lex.EXTENSION:
      results.append(handle_extension(lex.next()[1][2:-1]))
    elif lex.peek()[0] == lex.TEXT:
      results.append(lex.next()[1])
    elif lex.peek()[0] == lex.DOLLAR_DOLLAR_BRACE:
      results.append(lex.next()[1][1:])
  return ''.join(map(str, results))


# Expands macros, replaces properties, and evaluates expressions
def eval_all(root, macros, symbols):
  # Evaluates the attributes for the root node
  for at in root.attributes.items():
    result = eval_text(at[1], symbols)
    root.setAttribute(at[0], result)

  previous = root
  node = next_node(previous)
  while node:
    if node.nodeType == xml.dom.Node.ELEMENT_NODE:
      if node.tagName in macros:
        body = macros[node.tagName].cloneNode(deep=True)
        params = body.getAttribute('params').split()

        # Parse default values for any parameters
        defaultmap = {}
        for param in params[:]:
          splitParam = param.split(':=')

          if len(splitParam) == 2:
            defaultmap[splitParam[0]] = splitParam[1]
            params.remove(param)
            params.append(splitParam[0])

          elif len(splitParam) != 1:
            raise XacroException("Invalid parameter definition")

        # Expands the macro
        scoped = Table(symbols)
        for name, value in node.attributes.items():
          if not name in params:
            raise XacroException("Invalid parameter \"%s\" while expanding macro \"%s\"" %
                                 (str(name), str(node.tagName)))
          params.remove(name)
          scoped[name] = eval_text(value, symbols)

        # Pulls out the block arguments, in order
        cloned = node.cloneNode(deep=True)
        eval_all(cloned, macros, symbols)
        block = cloned.firstChild
        for param in params[:]:
          if param[0] == '*':
            while block and block.nodeType != xml.dom.Node.ELEMENT_NODE:
              block = block.nextSibling
            if not block:
              raise XacroException("Not enough blocks while evaluating macro %s" %
                                   str(node.tagName))
            params.remove(param)
            scoped[param] = block
            block = block.nextSibling

        # Try to load defaults for any remaining non-block parameters
        for param in params[:]:
          if param[0] != '*' and param in defaultmap:
            scoped[param] = defaultmap[param]
            params.remove(param)

        if params:
          raise XacroException("Parameters [%s] were not set for macro %s" %
                               (",".join(params), str(node.tagName)))
        eval_all(body, macros, scoped)

        # Replaces the macro node with the expansion
        for e in list(child_nodes(body)):  # Ew
          node.parentNode.insertBefore(e, node)
        node.parentNode.removeChild(node)

        node = None
      elif node.tagName == 'arg' or node.tagName == 'xacro:arg':
        name = node.getAttribute('name')
        if not name:
          raise XacroException("Argument name missing")
        default = node.getAttribute('default')
        if default and name not in substitution_args_context['arg']:
          substitution_args_context['arg'][name] = default

        node.parentNode.removeChild(node)
        node = None

      elif node.tagName == 'insert_block' or node.tagName == 'xacro:insert_block':
        name = node.getAttribute('name')

        if ("**" + name) in symbols:
          # Multi-block
          block = symbols['**' + name]

          for e in list(child_nodes(block)):
            node.parentNode.insertBefore(e.cloneNode(deep=True), node)
          node.parentNode.removeChild(node)
        elif ("*" + name) in symbols:
          # Single block
          block = symbols['*' + name]

          node.parentNode.insertBefore(block.cloneNode(deep=True), node)
          node.parentNode.removeChild(node)
        else:
          raise XacroException("Block \"%s\" was never declared" % name)

        node = None
      elif node.tagName in ['if', 'xacro:if', 'unless', 'xacro:unless']:
        value = eval_text(node.getAttribute('value'), symbols)
        try:
          if value == 'true': keep = True
          elif value == 'false': keep = False
          else: keep = float(value)
        except ValueError:
          raise XacroException(
              "Xacro conditional evaluated to \"%s\". Acceptable evaluations are one of [\"1\",\"true\",\"0\",\"false\"]"
              % value)
        if node.tagName in ['unless', 'xacro:unless']: keep = not keep
        if keep:
          for e in list(child_nodes(node)):
            node.parentNode.insertBefore(e.cloneNode(deep=True), node)

        node.parentNode.removeChild(node)
      else:
        # Evals the attributes
        for at in node.attributes.items():
          result = eval_text(at[1], symbols)
          node.setAttribute(at[0], result)
        previous = node
    elif node.nodeType == xml.dom.Node.TEXT_NODE:
      node.data = eval_text(node.data, symbols)
      previous = node
    else:
      previous = node

    node = next_node(previous)
  return macros


# Expands everything except includes
def eval_self_contained(doc):
  macros = grab_macros(doc)
  symbols = grab_properties(doc)
  eval_all(doc.documentElement, macros, symbols)


def print_usage(exit_code=0):
  print("Usage: %s [-o <output>] <input>" % 'xacro.py')
  print("       %s --deps       Prints dependencies" % 'xacro.py')
  print("       %s --includes   Only evalutes includes" % 'xacro.py')
  sys.exit(exit_code)


def set_substitution_args_context(context={}):
  substitution_args_context['arg'] = context


def open_output(output_filename):
  if output_filename is None:
    return sys.stdout
  else:
    return open(output_filename, 'w')


def main():
  try:
    opts, args = getopt.gnu_getopt(sys.argv[1:], "ho:", ['deps', 'includes'])
  except getopt.GetoptError as err:
    print(str(err))
    print_usage(2)

  just_deps = False
  just_includes = False

  output_filename = None
  for o, a in opts:
    if o == '-h':
      print_usage(0)
    elif o == '-o':
      output_filename = a
    elif o == '--deps':
      just_deps = True
    elif o == '--includes':
      just_includes = True

  if len(args) < 1:
    print("No input given")
    print_usage(2)

  # Process substitution args
  # set_substitution_args_context(load_mappings(sys.argv))
  set_substitution_args_context((sys.argv))

  f = open(args[0])
  doc = None
  try:
    doc = parse(f)
  except xml.parsers.expat.ExpatError:
    sys.stderr.write("Expat parsing error.  Check that:\n")
    sys.stderr.write(" - Your XML is correctly formed\n")
    sys.stderr.write(" - You have the xacro xmlns declaration: " +
                     "xmlns:xacro=\"http://www.ros.org/wiki/xacro\"\n")
    sys.stderr.write("\n")
    raise
  finally:
    f.close()

  process_includes(doc, os.path.dirname(args[0]))
  if just_deps:
    for inc in all_includes:
      sys.stdout.write(inc + " ")
    sys.stdout.write("\n")
  elif just_includes:
    doc.writexml(open_output(output_filename))
    print()
  else:
    eval_self_contained(doc)
    banner = [
        xml.dom.minidom.Comment(c) for c in [
            " %s " % ('=' * 83),
            " |    This document was autogenerated by xacro from %-30s | " % args[0],
            " |    EDITING THIS FILE BY HAND IS NOT RECOMMENDED  %-30s | " % "",
            " %s " % ('=' * 83)
        ]
    ]
    first = doc.firstChild
    for comment in banner:
      doc.insertBefore(comment, first)

    open_output(output_filename).write(doc.toprettyxml(indent='  '))
    print()


if __name__ == '__main__':
  main()



import math

NUM_VERTS_X = 30
NUM_VERTS_Y = 30
totalVerts = NUM_VERTS_X * NUM_VERTS_Y
totalTriangles = 2 * (NUM_VERTS_X - 1) * (NUM_VERTS_Y - 1)
offset = -50.0
TRIANGLE_SIZE = 1.
waveheight = 0.1
gGroundVertices = [None] * totalVerts * 3
gGroundIndices = [None] * totalTriangles * 3

i = 0

for i in range(NUM_VERTS_X):
  for j in range(NUM_VERTS_Y):
    gGroundVertices[(i + j * NUM_VERTS_X) * 3 + 0] = (i - NUM_VERTS_X * 0.5) * TRIANGLE_SIZE
    gGroundVertices[(i + j * NUM_VERTS_X) * 3 + 1] = (j - NUM_VERTS_Y * 0.5) * TRIANGLE_SIZE
    gGroundVertices[(i + j * NUM_VERTS_X) * 3 +
                    2] = waveheight * math.sin(float(i)) * math.cos(float(j) + offset)

index = 0
for i in range(NUM_VERTS_X - 1):
  for j in range(NUM_VERTS_Y - 1):
    gGroundIndices[index] = 1 + j * NUM_VERTS_X + i
    index += 1
    gGroundIndices[index] = 1 + j * NUM_VERTS_X + i + 1
    index += 1
    gGroundIndices[index] = 1 + (j + 1) * NUM_VERTS_X + i + 1
    index += 1
    gGroundIndices[index] = 1 + j * NUM_VERTS_X + i
    index += 1
    gGroundIndices[index] = 1 + (j + 1) * NUM_VERTS_X + i + 1
    index += 1
    gGroundIndices[index] = 1 + (j + 1) * NUM_VERTS_X + i
    index += 1

#print(gGroundVertices)
#print(gGroundIndices)

print("o Terrain")

for i in range(totalVerts):
  print("v "),
  print(gGroundVertices[i * 3 + 0]),
  print(" "),
  print(gGroundVertices[i * 3 + 1]),
  print(" "),
  print(gGroundVertices[i * 3 + 2])

for i in range(totalTriangles):
  print("f "),
  print(gGroundIndices[i * 3 + 0]),
  print(" "),
  print(gGroundIndices[i * 3 + 1]),
  print(" "),
  print(gGroundIndices[i * 3 + 2]),
  print(" ")
